﻿#include "define.h"

// INCLUDE
	vec4 computePixelColor();
// IN
	uniform uint  shared_pool_size;
#if multisample
	layout(binding = 0, r32ui) coherent uniform uimage2DMS   image_next;
	layout(binding = 1, r32ui) coherent uniform uimage2DMS   image_counter;
#else
	layout(binding = 0, r32ui) coherent uniform uimage2DRect image_next;
	layout(binding = 1, r32ui) coherent uniform uimage2DRect image_counter;
#endif 
	layout(binding = 3, rg32f) writeonly uniform  imageBuffer image_peel;
	layout(binding = 4, r32ui) writeonly uniform uimageBuffer image_pointers;
	layout(binding = 5, offset = 0)		 uniform atomic_uint  next_address;

#if multisample
	void addPixelFragCounter	   (ivec2 coords, uint val)	{		 imageAtomicAdd		(image_counter, coords, gl_SampleID, val);}
	uint exchangePixelCurrentPageID(ivec2 coords, uint val)	{ return imageAtomicExchange(image_next   , coords, gl_SampleID, val);}
#else
	void addPixelFragCounter	   (ivec2 coords, uint val)	{		 imageAtomicAdd		(image_counter, coords, val);}
	uint exchangePixelCurrentPageID(ivec2 coords, uint val)	{ return imageAtomicExchange(image_next   , coords, val);}
#endif 
	void sharedPoolSetValue(uint index, vec4 val)			{ imageStore(image_peel	   , (int)index, val);}
	void sharedPoolSetLink (uint id   , uint val)			{ imageStore(image_pointers, (int)id   , uvec4(val, 0U, 0U, 0U) );}

void main(void)
{
	ivec2 coords  = ivec2(gl_FragCoord.xy);
	uint  page_id = atomicCounterIncrement(next_address);

	vec4  value = vec4(packUnorm4x8(computePixelColor().abgr), gl_FragCoord.z, 0.0f, 0.0f);
	if(page_id < shared_pool_size)
	{
		addPixelFragCounter(coords , 1U);
		sharedPoolSetLink  (page_id, exchangePixelCurrentPageID(coords, page_id));
		sharedPoolSetValue (page_id, value);

		discard;
	}
}